/////////////////////////////////////////////////////////////////////////////////
//
// B2APIEx.cpp
//
// Copyright (C) 2000 YUGA. All rights reserved.
//
/*
 * $Log: B2APIEx.cpp,v $
 */


/////////////////////////////////////////////////////////////////////////////////
//
#include "B2APIEx.h"
#include "exception"


#pragma warning(push)
#pragma warning(disable:4101)
#pragma warning(disable:4786)


/////////////////////////////////////////////////////////////////////////////////
//
B2API* B2APIEx::b2api = NULL;
B2APIEx* B2APIEx::instance = NULL;
CriticalSection B2APIEx::critsec;

HWND B2APIEx::b2main;
HWND B2APIEx::b2tree;
HWND B2APIEx::b2list;
HWND B2APIEx::b2view;
MessageMap B2APIEx::message;

ConfigMgr* B2APIEx::configMgr;

string* B2APIEx::std_string_out = NULL;
LPCSTR B2APIEx::const_string_param1 = NULL;
LPCSTR B2APIEx::const_string_param2 = NULL;
LPSTR B2APIEx::string_param1 = NULL;
LPSTR B2APIEx::string_param2 = NULL;
HWND B2APIEx::window_param1 = NULL;
int B2APIEx::integer_param1 = 0;
int B2APIEx::integer_param2 = 0;
Config* B2APIEx::config_param1 = NULL;


/////////////////////////////////////////////////////////////////////////////////
//
B2APIEx::B2APIEx(ConfigMgr* initConfigMgr)
{
    configMgr = initConfigMgr;
}


B2APIEx::~B2APIEx()
{
    instance = NULL;
}


B2API* B2APIEx::Instance()
{
    b2api = B2API::Instance();
    return b2api;
}


B2APIEx* B2APIEx::InstanceEx(ConfigMgr* configMgr)
{
    critsec.enter();
    if (instance == NULL) {
        instance = new B2APIEx(configMgr);
        if (instance && !instance->init()) {
            delete instance;
            instance = NULL;
        }
    }
    critsec.leave();
    return instance;
}


B2APIEx* B2APIEx::InstanceEx()
{
    return instance;
}


bool B2APIEx::init()
{
    if (!b2api->GetWindowHandles(&b2main, &b2tree, &b2list, &b2view)) {
        return false;
    }

    DWORD id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcGetSource);
    message[B2APIEX_GETSOURCE] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcGetText);
    message[B2APIEX_GETTEXT] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcGetHeader);
    message[B2APIEX_GETHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompGetSource);
    message[B2APIEX_COMPGETSOURCE] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompGetText);
    message[B2APIEX_COMPGETTEXT] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompGetHeader);
    message[B2APIEX_COMPGETHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcSetSource);
    message[B2APIEX_SETSOURCE] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcSetText);
    message[B2APIEX_SETTEXT] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompSetSource);
    message[B2APIEX_COMPSETSOURCE] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompSetText);
    message[B2APIEX_COMPSETTEXT] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcGetSpecifiedHeader);
    message[B2APIEX_GETSPECIFIEDHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcSetSpecifiedHeader);
    message[B2APIEX_SETSPECIFIEDHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompGetSpecifiedHeader);
    message[B2APIEX_COMPGETSPECIFIEDHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_COMPOSE, B2APIEx::CmdProcCompSetSpecifiedHeader);
    message[B2APIEX_COMPSETSPECIFIEDHEADER] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcGetConfig);
    message[B2APIEX_GETCONFIG] = id;
    id = b2api->RegisterCommand("", BKC_MENU_MAIN, B2APIEx::CmdProcSetConfig);
    message[B2APIEX_SETCONFIG] = id;

	return true;
}


bool B2APIEx::equalThread(HWND hWnd)
{
    DWORD id1 = ::GetWindowThreadProcessId(hWnd, NULL);
    DWORD id2 = ::GetCurrentThreadId();

    if (id1 != 0 && id1 != id2) {
        return false;
    }
    return true;
}


/////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////
//
string B2APIEx::GetSource(LPCSTR lpMailID)
{
    string result;

    if (equalThread(b2main)) {
        LPSTR source = b2api->GetSource(lpMailID);
        result = source;
        b2api->Free(source);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        const_string_param1 = lpMailID;

        ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_GETSOURCE], 0);

        std_string_out = NULL;
        const_string_param1 = NULL;
        critsec.leave();
    }

    return result;
}


string B2APIEx::GetText(LPSTR lpszMimeType, int nBuf)
{
    string result;

    if (equalThread(b2main)) {
        LPSTR text = b2api->GetText(lpszMimeType, nBuf);
        result = text;
        b2api->Free(text);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        string_param1 = lpszMimeType;
        integer_param1 = nBuf;

        ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_GETTEXT], 0);

        std_string_out = NULL;
        string_param1 = NULL;
        integer_param1 = 0;
        critsec.leave();
    }

    return result;
}


string B2APIEx::GetHeader(LPCSTR lpMailID)
{
    string result;

    if (equalThread(b2main)) {
        LPSTR header = b2api->GetHeader(lpMailID);
        result = header;
        b2api->Free(header);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        const_string_param1 = lpMailID;

        ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_GETHEADER], 0);

        std_string_out = NULL;
        const_string_param1 = NULL;
        critsec.leave();
    }

    return result;
}


string B2APIEx::CompGetSource(HWND hWnd)
{
    string result;

    if (equalThread(hWnd)) {
        LPSTR source = b2api->CompGetSource(hWnd);
        result = source;
        b2api->Free(source);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        window_param1 = hWnd;

        ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPGETSOURCE], 0);

        std_string_out = NULL;
        window_param1 = NULL;
        critsec.leave();
    }

    return result;
}


string B2APIEx::CompGetText(HWND hWnd, LPSTR lpszMimeType, int nBuf)
{
    string result;

    if (equalThread(hWnd)) {
        LPSTR text = b2api->CompGetText(hWnd, lpszMimeType, nBuf);
        result = text;
        b2api->Free(text);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        window_param1 = hWnd;
        const_string_param1 = lpszMimeType;
        integer_param1 = nBuf;

        ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPGETTEXT], 0);

        std_string_out = NULL;
        window_param1 = NULL;
        const_string_param1 = NULL;
        integer_param1 = 0;
        critsec.leave();
    }

    return result;
}


string B2APIEx::CompGetHeader(HWND hWnd)
{
    string result;

    if (equalThread(hWnd)) {
        LPSTR header = b2api->CompGetHeader(hWnd);
        result = header;
        b2api->Free(header);
    }
    else {
        critsec.enter();
        std_string_out = &result;
        window_param1 = hWnd;

        ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPGETHEADER], 0);

        std_string_out = NULL;
        window_param1 = NULL;
        critsec.leave();
    }

    return result;
}


////////////////////////////////////////////////////////////////////////
//
void B2APIEx::SetSource(LPCSTR lpMailID, LPCSTR lpSource)
{
    if (equalThread(b2main)) {
        b2api->SetSource(lpMailID, lpSource);
        return;
    }

    critsec.enter();
    const_string_param1 = lpMailID;
    const_string_param2 = lpSource;

    ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_SETSOURCE], 0);

    const_string_param1 = NULL;
    const_string_param2 = NULL;
    critsec.leave();
}


void B2APIEx::SetText(int nMode, LPCSTR lpText)
{
    if (equalThread(b2main)) {
        b2api->SetText(nMode, lpText);
        return;
    }

    critsec.enter();
    integer_param1 = nMode;
    const_string_param1 = lpText;

    ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_SETTEXT], 0);

    integer_param1 = 0;
    const_string_param1 = NULL;
    critsec.leave();
}


void B2APIEx::CompSetSource(HWND hWnd, LPCSTR lpSource)
{
    if (equalThread(hWnd)) {
        b2api->CompSetSource(hWnd, lpSource);
        return;
    }

    critsec.enter();
    window_param1 = hWnd;
    const_string_param1 = lpSource;

    ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPSETSOURCE], 0);

    window_param1 = NULL;
    const_string_param1 = NULL;
    critsec.leave();
}


void B2APIEx::CompSetText(HWND hWnd, int nMode, LPCSTR lpText)
{
    if (equalThread(hWnd)) {
        b2api->CompSetText(hWnd, nMode, lpText);
        return;
    }

    critsec.enter();
    window_param1 = hWnd;
    integer_param1 = nMode;
    const_string_param1 = lpText;

    ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPSETTEXT], 0);

    window_param1 = NULL;
    integer_param1 = 0;
    const_string_param1 = NULL;
    critsec.leave();
}


////////////////////////////////////////////////////////////////////////
//
void B2APIEx::GetSpecifiedHeader(LPCSTR lpHeader, LPSTR lpszData, int nBuf)
{
    if (equalThread(b2main)) {
        b2api->GetSpecifiedHeader(lpHeader, lpszData, nBuf);
        return;
    }

    critsec.enter();
    const_string_param1 = lpHeader;
    string_param1 = lpszData;
    integer_param1 = nBuf;

    ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_GETSPECIFIEDHEADER], 0);

    const_string_param1 = NULL;
    string_param1 = NULL;
    integer_param1 = 0;
    critsec.leave();
}


void B2APIEx::SetSpecifiedHeader(LPCSTR lpHeader, LPCSTR lpszData)
{
    if (equalThread(b2main)) {
        b2api->SetSpecifiedHeader(lpHeader, lpszData);
        return;
    }

    critsec.enter();
    const_string_param1 = lpHeader;
    const_string_param2 = lpszData;

    ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_SETSPECIFIEDHEADER], 0);

    const_string_param1 = NULL;
    const_string_param2 = NULL;
    critsec.leave();
}


void B2APIEx::CompGetSpecifiedHeader(HWND hWnd, LPCSTR lpHeader, LPSTR lpszData, int nBuf)
{
    if (equalThread(b2main)) {
        b2api->CompGetSpecifiedHeader(hWnd, lpHeader, lpszData, nBuf);
        return;
    }

    critsec.enter();
    window_param1 = hWnd;
    const_string_param1 = lpHeader;
    string_param1 = lpszData;
    integer_param1 = nBuf;

    ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPGETSPECIFIEDHEADER], 0);

    window_param1 = NULL;
    const_string_param1 = NULL;
    string_param1 = NULL;
    integer_param1 = 0;
    critsec.leave();
}


void B2APIEx::CompSetSpecifiedHeader(HWND hWnd, LPCSTR lpHeader, LPCSTR lpszData)
{
    if (equalThread(b2main)) {
        b2api->CompSetSpecifiedHeader(hWnd, lpHeader, lpszData);
        return;
    }

    critsec.enter();
    window_param1 = hWnd;
    const_string_param1 = lpHeader;
    const_string_param2 = lpszData;

    ::SendMessage(hWnd, WM_COMMAND, message[B2APIEX_COMPSETSPECIFIEDHEADER], 0);

    window_param1 = NULL;
    const_string_param1 = NULL;
    const_string_param2 = NULL;
    critsec.leave();
}


////////////////////////////////////////////////////////////////////////
//
Config B2APIEx::GetConfig()
{
    Config config;

    if (equalThread(b2main)) {
        config_param1 = &config;
        CmdProcGetConfig(b2main, reinterpret_cast<LPARAM>(&config));
    } else {
        critsec.enter();
        config_param1 = &config;
        ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_GETCONFIG], 0);
        critsec.leave();
    }

    return config;
}


void B2APIEx::SetConfig(const Config& config)
{
    if (equalThread(b2main)) {
        config_param1 = const_cast<Config*>(&config);
        CmdProcSetConfig(b2main, 0);
    } else {
        critsec.enter();
        config_param1 = const_cast<Config*>(&config);
        ::SendMessage(b2main, WM_COMMAND, message[B2APIEX_SETCONFIG], 0);
        critsec.leave();
    }
}


////////////////////////////////////////////////////////////////////////
//
HWND B2APIEx::GetMainWindowHandle()
{
    return b2main;
}


HWND B2APIEx::GetTreeWindowHandle()
{
    return b2tree;
}


HWND B2APIEx::GetListWindowHandle()
{
    return b2list;
}


HWND B2APIEx::GetViewWindowHandle()
{
    return b2view;
}


/////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////
//
void WINAPI B2APIEx::CmdProcGetSource(HWND hWnd, LPARAM lParam)
{
    LPSTR source = b2api->GetSource(const_string_param1);
    *std_string_out = source;
    b2api->Free(source);
}


void WINAPI B2APIEx::CmdProcGetText(HWND hWnd, LPARAM lParam)
{
    LPSTR text = b2api->GetText(string_param1, integer_param1);
    *std_string_out = text;
    b2api->Free(text);
}


void WINAPI B2APIEx::CmdProcGetHeader(HWND hWnd, LPARAM lParam)
{
    LPSTR header = b2api->GetHeader(const_string_param1);
    *std_string_out = header;
    b2api->Free(header);
}


void WINAPI B2APIEx::CmdProcCompGetSource(HWND hWnd, LPARAM lParam)
{
    LPSTR source = b2api->CompGetSource(window_param1);
    *std_string_out = source;
    b2api->Free(source);
}


void WINAPI B2APIEx::CmdProcCompGetText(HWND hWnd, LPARAM lParam)
{
    LPSTR text = b2api->CompGetText(window_param1, string_param1, integer_param1);
    *std_string_out = text;
    b2api->Free(text);
}


void WINAPI B2APIEx::CmdProcCompGetHeader(HWND hWnd, LPARAM lParam)
{
    LPSTR header = b2api->CompGetHeader(window_param1);
    *std_string_out = header;
    b2api->Free(header);
}


////////////////////////////////////////////////////////////////////////
//
void WINAPI B2APIEx::CmdProcSetSource(HWND hWnd, LPARAM lParam)
{
    b2api->SetSource(const_string_param1, const_string_param2);
}


void WINAPI B2APIEx::CmdProcSetText(HWND hWnd, LPARAM lParam)
{
    b2api->SetText(integer_param1, const_string_param1);
}


void WINAPI B2APIEx::CmdProcCompSetSource(HWND hWnd, LPARAM lParam)
{
    BOOL accept = ::IsWindowEnabled(window_param1);
    ::EnableWindow(window_param1, TRUE);
    b2api->CompSetSource(window_param1, const_string_param1);
    ::EnableWindow(window_param1, accept);
}


void WINAPI B2APIEx::CmdProcCompSetText(HWND hWnd, LPARAM lParam)
{
    b2api->CompSetText(window_param1, integer_param1, const_string_param1);
}


////////////////////////////////////////////////////////////////////////
//
void WINAPI B2APIEx::CmdProcGetSpecifiedHeader(HWND hWnd, LPARAM lParam)
{
    b2api->GetSpecifiedHeader(const_string_param1, string_param1, integer_param1);
}


void WINAPI B2APIEx::CmdProcSetSpecifiedHeader(HWND hWnd, LPARAM lParam)
{
    b2api->SetSpecifiedHeader(const_string_param1, const_string_param2);
}


void WINAPI B2APIEx::CmdProcCompGetSpecifiedHeader(HWND hWnd, LPARAM lParam)
{
    b2api->CompGetSpecifiedHeader(window_param1, const_string_param1, string_param1, integer_param1);
}


void WINAPI B2APIEx::CmdProcCompSetSpecifiedHeader(HWND hWnd, LPARAM lParam)
{
    b2api->CompSetSpecifiedHeader(window_param1, const_string_param1, const_string_param2);
}


////////////////////////////////////////////////////////////////////////
//
void WINAPI B2APIEx::CmdProcGetConfig(HWND hWnd, LPARAM lParam)
{
    *config_param1 = configMgr->get();
}


void WINAPI B2APIEx::CmdProcSetConfig(HWND hWnd, LPARAM lParam)
{
    configMgr->set(*config_param1);
}


/////////////////////////////////////////////////////////////////////////////////
//
#pragma warning(pop)