/**
 * @file Config.cpp
 * @brief IvVݒ̏NX̎
 * @author  Copyright (C) 2002 YUGA <yuga@ops.dti.ne.jp>
 * $Date: $
 * $Revision: $
 *
 */

#include "Config.h"


/////////////////////////////////////////////////////////////////////////////////
//
using namespace std;


/////////////////////////////////////////////////////////////////////////////////
//
Config::Config()
{
}


Config::Config(const Config& initConfig)
{
    encryptDefaultKey = initConfig.encryptDefaultKey;
    saveDecryptPassphrase = initConfig.saveDecryptPassphrase;
    stDecrypt = initConfig.stDecrypt;
    saveSignPassphrase = initConfig.saveSignPassphrase;
    stSign = initConfig.stSign;

    usePGPlog = initConfig.usePGPlog;
    displayVerificationResultOnListView = initConfig.displayVerificationResultOnListView;
    dtVerificationResult = initConfig.dtVerificationResult;
    dcVerificationCorrect = initConfig.dcVerificationCorrect;
    dcVerificationInCorrect = initConfig.dcVerificationInCorrect;
    eraseBackgroundOfListView = initConfig.eraseBackgroundOfListView;
}


Config::~Config()
{
}


Config& Config::operator=(const Config& newConfig)
{
    encryptDefaultKey = newConfig.encryptDefaultKey;
    saveDecryptPassphrase = newConfig.saveDecryptPassphrase;
    stDecrypt = newConfig.stDecrypt;
    saveSignPassphrase = newConfig.saveSignPassphrase;
    stSign = newConfig.stSign;

    usePGPlog = newConfig.usePGPlog;
    displayVerificationResultOnListView = newConfig.displayVerificationResultOnListView;
    dtVerificationResult = newConfig.dtVerificationResult;
    dcVerificationCorrect = newConfig.dcVerificationCorrect;
    dcVerificationInCorrect = newConfig.dcVerificationInCorrect;
    eraseBackgroundOfListView = newConfig.eraseBackgroundOfListView;

    return *this;
}


bool Config::isEncryptDefaultKey() const
{
    return encryptDefaultKey;
}


void Config::setEncryptDefaultKey(bool use)
{
    encryptDefaultKey = use;
}


bool Config::isSaveDecryptPassphrase()
{
    return saveDecryptPassphrase;
}


void Config::setSaveDecryptPassphrase(bool save)
{
    saveDecryptPassphrase = save;
}


const SYSTEMTIME& Config::getSaveTimeDecryptPassphrase() const
{
    return stDecrypt;
}


void Config::setSaveTimeDecryptPassphrase(const SYSTEMTIME& time)
{
    stDecrypt = time;
}


bool Config::isSaveSignPassphrase()
{
    return saveSignPassphrase;
}


void Config::setSaveSignPassphrase(bool save)
{
    saveSignPassphrase = save;
}


const SYSTEMTIME& Config::getSaveTimeSignPassphrase() const
{
    return stSign;
}


void Config::setSaveTimeSignPassphrase(const SYSTEMTIME& time)
{
    stSign = time;
}


bool Config::isUsePGPlog()
{
    return usePGPlog;
}


bool Config::isDisplayVerificationResultOnListView()
{
    return displayVerificationResultOnListView;
}


DWORD Config::getDisplayTimeVerificationResult()
{
    return dtVerificationResult;
}


const COLORREF& Config::getDisplayColorVerificationCorrect()
{
    return dcVerificationCorrect;
}


const COLORREF& Config::getDisplayColorVerificationInCorrect()
{
    return dcVerificationInCorrect;
}


bool Config::isEraseBackgroundOfListView()
{
    return eraseBackgroundOfListView;
}


void Config::setUsePGPlog(bool use)
{
    usePGPlog = use;
}


void Config::setDisplayVerificationResultOnListView(bool display)
{
    displayVerificationResultOnListView = display;
}


void Config::setDisplayTimeVerificationResult(DWORD time)
{
    dtVerificationResult = time;
}


void Config::setDisplayColorVerificationCorrect(const COLORREF& color)
{
    dcVerificationCorrect = color;
}


void Config::setDisplayColorVerificationInCorrect(const COLORREF& color)
{
    dcVerificationInCorrect = color;
}


void Config::setEraseBackgroundOfListView(bool erase)
{
    eraseBackgroundOfListView = erase;
}


/////////////////////////////////////////////////////////////////////////////////
//
ConfigMgr::ConfigMgr(const string& fullPath)
{
    iniFilePath = fullPath;
}


ConfigMgr::~ConfigMgr()
{
}


void ConfigMgr::save()
{
    char szVal[60];

    sprintf(szVal, "%d", (config.isEncryptDefaultKey() ? TRUE:FALSE));
    WritePrivateProfileString("Settings", "encryptDefaultKey", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", (config.isSaveDecryptPassphrase() ? TRUE:FALSE));
    WritePrivateProfileString("Settings", "saveDecryptPassphrase", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeDecryptPassphrase().wHour);
    WritePrivateProfileString("Settings", "stDecryptHour", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeDecryptPassphrase().wMinute);
    WritePrivateProfileString("Settings", "stDecryptMinute", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeDecryptPassphrase().wSecond);
    WritePrivateProfileString("Settings", "stDecryptSecond", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", (config.isSaveSignPassphrase() ? TRUE:FALSE));
    WritePrivateProfileString("Settings", "saveSignPassphrase", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeSignPassphrase().wHour);
    WritePrivateProfileString("Settings", "stSignHour", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeSignPassphrase().wMinute);
    WritePrivateProfileString("Settings", "stSignMinute", szVal, iniFilePath.c_str());

    sprintf(szVal, "%d", config.getSaveTimeSignPassphrase().wSecond);
    WritePrivateProfileString("Settings", "stSignSecond", szVal, iniFilePath.c_str());

}


void ConfigMgr::load()
{
    string::size_type pos = iniFilePath.find_last_of("\\");
    ::CreateDirectory(iniFilePath.substr(0, pos).c_str(), NULL);

    // Settings
    bool encryptDefaultKey;         // Default KeygĈÍ邩H
    bool saveDecryptPassphrase;     // ppXt[YLbV邩H
    SYSTEMTIME stDecrypt;           // ppXt[ỸLbV
    bool saveSignPassphrase;        // ppXt[YLbV邩H
    SYSTEMTIME stSign;              // ppXt[ỸLbV

    encryptDefaultKey = (::GetPrivateProfileInt("Settings", "encryptDefaultKey", TRUE, iniFilePath.c_str()) == TRUE);

    saveDecryptPassphrase= (::GetPrivateProfileInt("Settings", "saveDecryptPassphrase", FALSE, iniFilePath.c_str()) == TRUE);

    ::GetSystemTime(&stDecrypt);
    stDecrypt.wHour   = ::GetPrivateProfileInt("Settings", "stDecryptHour",   0, iniFilePath.c_str());
    stDecrypt.wMinute = ::GetPrivateProfileInt("Settings", "stDecryptMinute", 2, iniFilePath.c_str());
    stDecrypt.wSecond = ::GetPrivateProfileInt("Settings", "stDecryptSecond", 0, iniFilePath.c_str());

    saveSignPassphrase = (::GetPrivateProfileInt("Settings", "saveSignPassphrase", FALSE, iniFilePath.c_str()) == TRUE);

    ::GetSystemTime(&stSign);
    stSign.wHour   = ::GetPrivateProfileInt("Settings", "stSignHour",   0, iniFilePath.c_str());
    stSign.wMinute = ::GetPrivateProfileInt("Settings", "stSignMinute", 2, iniFilePath.c_str());
    stSign.wSecond = ::GetPrivateProfileInt("Settings", "stSignSecond", 0, iniFilePath.c_str());

    config.setEncryptDefaultKey(encryptDefaultKey);
    config.setSaveDecryptPassphrase(saveDecryptPassphrase);
    config.setSaveTimeDecryptPassphrase(stDecrypt);
    config.setSaveSignPassphrase(saveSignPassphrase);
    config.setSaveTimeSignPassphrase(stSign);

    // ExpertSettings
    bool usePGPlog;                             // PGPloggp邩H
    bool displayVerificationResultOnListView;   // ،ʂXgr[ɕ\邩H
    DWORD dtVerificationResult;                 // ،ʂXgr[ɕ\鎞
    BYTE dcVerificationCorrectRed;              // ،"Good Signature"̐FF
    BYTE dcVerificationCorrectGreen;            // ،"Good Signature"̐FF
    BYTE dcVerificationCorrectBlue;             // ،"Good Signature"̐FF
    BYTE dcVerificationInCorrectRed;            // ،"Bad Signature"̐FF
    BYTE dcVerificationInCorrectGreen;          // ،"Bad Signature"̐FF
    BYTE dcVerificationInCorrectBlue;           // ،"Bad Signature"̐FF
    bool eraseBackgroundOfListView;             // ،ʂ\ƂXg[̔wi邩H

    usePGPlog = (::GetPrivateProfileInt("ExpertSettings", "usePGPlog", TRUE, iniFilePath.c_str()) == TRUE);
    displayVerificationResultOnListView = (::GetPrivateProfileInt("ExpertSettings", "displayVerificationResultOnListView", TRUE, iniFilePath.c_str()) == TRUE);
    dtVerificationResult = ::GetPrivateProfileInt("ExpertSettings", "dtVerificationResult", 3000, iniFilePath.c_str());
    dcVerificationCorrectRed     = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationCorrectRed", 0, iniFilePath.c_str());
    dcVerificationCorrectGreen   = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationCorrectGreen", 0, iniFilePath.c_str());
    dcVerificationCorrectBlue    = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationCorrectBlue", 255, iniFilePath.c_str());
    dcVerificationInCorrectRed   = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationInCorrectRed", 255, iniFilePath.c_str());
    dcVerificationInCorrectGreen = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationInCorrectGreen", 0, iniFilePath.c_str());
    dcVerificationInCorrectBlue  = ::GetPrivateProfileInt("ExpertSettings", "dcVerificationInCorrectBlue", 0, iniFilePath.c_str());
    eraseBackgroundOfListView = (::GetPrivateProfileInt("ExpertSettings", "eraseBackgroundOfListView", TRUE, iniFilePath.c_str()) == TRUE);

    config.setUsePGPlog(usePGPlog);
    config.setDisplayVerificationResultOnListView(displayVerificationResultOnListView);
    config.setDisplayTimeVerificationResult(dtVerificationResult);
    config.setDisplayColorVerificationCorrect(RGB(dcVerificationCorrectRed,   dcVerificationCorrectGreen,   dcVerificationCorrectBlue));
    config.setDisplayColorVerificationInCorrect(RGB(dcVerificationInCorrectRed, dcVerificationInCorrectGreen, dcVerificationInCorrectBlue));
    config.setEraseBackgroundOfListView(eraseBackgroundOfListView);
}


Config ConfigMgr::get()
{
    return config;
}


void ConfigMgr::set(const Config& newConfig)
{
    config = newConfig;
}


/////////////////////////////////////////////////////////////////////////////////
//
