/*!
 * @file Hook.h
 * @brief Becky!WindowHookĂ낢낷B
 * @author  Copyright (C) 2002 YUGA <yuga@ops.dti.ne.jp>
 * $Date: $
 * $Revision: $
 *
 */


/////////////////////////////////////////////////////////////////////////////////
//
#include "Hook.h"


/////////////////////////////////////////////////////////////////////////////////
//
#define PGPB2_CORRECT_STATE_NONE      0
#define PGPB2_CORRECT_STATE_CORRECT   1
#define PGPB2_CORRECT_STATE_INCORRECT 2

#define PGPB2_CORRECT_PAINT_NONE      0
#define PGPB2_CORRECT_PAINT_PAINTED   1
#define PGPB2_CORRECT_PAINT_INVALID   2


/////////////////////////////////////////////////////////////////////////////////
//
HWND b2ListWindowHandle = 0;
DWORD b2viewThreadId = 0;
HHOOK hhook = 0;
WNDPROC oldWndProc = 0;
int state = PGPB2_CORRECT_STATE_NONE;
int paint = PGPB2_CORRECT_PAINT_NONE;
UINT_PTR event = 0;
Config config;
bool subclass = false;
B2APIEx* apiex = NULL;

/////////////////////////////////////////////////////////////////////////////////
//
void WINAPI HookStart(HINSTANCE hInstance)
{
    apiex = B2APIEx::InstanceEx();
    config = apiex->GetConfig();

    b2ListWindowHandle = apiex->GetListWindowHandle();

    // get thread ID of ListView
	b2viewThreadId = GetWindowThreadProcessId(b2ListWindowHandle, NULL);
	
	// install an hook to thread of ListView
	hhook = SetWindowsHookEx(WH_CALLWNDPROC, reinterpret_cast<HOOKPROC>(CallWndProc),
                hInstance, b2viewThreadId);

    if (!hhook) {
        MessageBox(apiex->GetMainWindowHandle(), "The error occured in SetWindowsHookEx.", "PGPB2 - Error", MB_OK | MB_ICONERROR);
    }
}


void WINAPI HookEnd()
{
    if (!subclass) {
        return;
    }

    SetWindowLong(b2ListWindowHandle, GWL_WNDPROC, (LONG)oldWndProc);
	oldWndProc = NULL;
    subclass = false;

    // uninstall my hook
    if(hhook != NULL) {
		UnhookWindowsHookEx(hhook);
        hhook = NULL;
    }

    // refresh the clock
    Invalidate();
}


/////////////////////////////////////////////////////////////////////////////////
//
LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	LPCWPSTRUCT pcwps = reinterpret_cast<LPCWPSTRUCT>(lParam);

	if (!subclass)	{
	    if(nCode >= 0 && pcwps && pcwps->hwnd) {
			// initialize  cf. wndproc.c
			InitCorrect(pcwps->hwnd);
		}
	}
	
	return CallNextHookEx(hhook, nCode, wParam, lParam);
}


/////////////////////////////////////////////////////////////////////////////////
//
void InitCorrect(HWND hWnd)
{
	oldWndProc = reinterpret_cast<WNDPROC>(SetWindowLong(b2ListWindowHandle, GWL_WNDPROC, reinterpret_cast<LONG>(WndProc)));
    subclass = (oldWndProc != 0);

    if (!subclass) {
        MessageBox(apiex->GetMainWindowHandle(), "The error occured in SetWindowLong.", "PGPB2 - Error", MB_OK | MB_ICONERROR);
    }
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    #define WM_XBUTTONDOWN 0x020B 
    
    switch(message)
	{
    case WM_KEYDOWN:
    case WM_LBUTTONDOWN:
    case WM_RBUTTONDOWN:
    case WM_MBUTTONDOWN:
    case WM_XBUTTONDOWN:
    case WM_VSCROLL:
        {
            if (paint == PGPB2_CORRECT_PAINT_PAINTED) {
                Invalidate();
            }
        }
        break;
    case WM_ERASEBKGND:
        {
            if (state != PGPB2_CORRECT_STATE_NONE && !config.isEraseBackgroundOfListView()) {
                return 0;
            }
        }
        break;
    case WM_SIZE:
        {
            if (state == PGPB2_CORRECT_PAINT_PAINTED) {
    	        InvalidateRect(b2ListWindowHandle, NULL, TRUE);
            }
        }
        break;
	case WM_PAINT:
	    {
            if (state == PGPB2_CORRECT_STATE_NONE) {
                break;
            }

            PAINTSTRUCT ps;
		    HDC hdc = BeginPaint(hwnd, &ps);
            char* message;
            if (state == PGPB2_CORRECT_STATE_CORRECT) {
                SetTextColor(hdc, config.getDisplayColorVerificationCorrect());
                message = "Good Signature";
            } else {
                SetTextColor(hdc, config.getDisplayColorVerificationInCorrect());
                message = "Bad Signature";
            }
            
            RECT rect;
            GetClientRect(hwnd, &rect);
            int height = rect.bottom / 3;
            int width = rect.right / (strlen(message) + 2);

            HFONT font = CreateFont(
                            height,                     // tHg̍
                            width,                      // ϕ
                            0,                          // ̊px
                            0,                          // x[XC̊px
                            FW_NORMAL,                  // tHg̑
                            0,                          // Α̂ɂ邩ǂ
                            0,                          // t邩ǂ
                            0,                          // t邩ǂ
                            ANSI_CHARSET,               // Zbg̎ʎq
                            OUT_OUTLINE_PRECIS,         // o͐x
                            CLIP_CHARACTER_PRECIS,      // NbsOx
                            ANTIALIASED_QUALITY,        // o͕i
                            DEFAULT_PITCH | FF_ROMAN,   // sb`ƃt@~
                            "Times New Roman"           // tHg
                            );

            HGDIOBJ oldFont = SelectObject(hdc, (HGDIOBJ) font);
            SetBkMode(hdc, TRANSPARENT);
            DrawText(hdc, message, strlen(message), &rect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
            SelectObject(hdc, oldFont);
		    EndPaint(hwnd, &ps);
            paint = PGPB2_CORRECT_PAINT_PAINTED;
		    return 0;
        }
        break;
    default:
        break;
    }
    return CallWindowProc(oldWndProc, hwnd, message, wParam, lParam);
}


/////////////////////////////////////////////////////////////////////////////////
//
void SetCorrect()
{
    if (!subclass) {
        return;
    }

    state = PGPB2_CORRECT_STATE_CORRECT;
    paint = PGPB2_CORRECT_PAINT_NONE;
	PostMessage(b2ListWindowHandle, WM_SIZE, SIZE_RESTORED, 0);
	InvalidateRect(b2ListWindowHandle, NULL, TRUE);

    if (event) {
        KillTimer(NULL, event);
    }
    event = SetTimer(NULL, NULL, config.getDisplayTimeVerificationResult(), EventHandler);
}


void SetInCorrect()
{
    if (!subclass) {
        return;
    }

    state = PGPB2_CORRECT_STATE_INCORRECT;
    paint = PGPB2_CORRECT_PAINT_NONE;
	PostMessage(b2ListWindowHandle, WM_SIZE, SIZE_RESTORED, 0);
	InvalidateRect(b2ListWindowHandle, NULL, TRUE);

    if (event) {
        KillTimer(NULL, event);
    }
    event = SetTimer(NULL, NULL, config.getDisplayTimeVerificationResult(), EventHandler);
}


void CALLBACK EventHandler(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
    if (event == idEvent) {
        event = 0;
        Invalidate();
    }
}


/////////////////////////////////////////////////////////////////////////////////
//
void Invalidate()
{
    if (!subclass || state == PGPB2_CORRECT_STATE_NONE) {
        return;
    }

    if (event) {
        KillTimer(NULL, event);
        event = 0;
    }

    state = PGPB2_CORRECT_STATE_NONE;
    paint = PGPB2_CORRECT_PAINT_INVALID;
	PostMessage(b2ListWindowHandle, WM_SIZE, SIZE_RESTORED, 0);
	InvalidateRect(b2ListWindowHandle, NULL, TRUE);
}


/////////////////////////////////////////////////////////////////////////////////
//
